#=======================================================================
#	$Id: HtPasswd.pm,v 1.1 2005/05/16 12:41:34 pythontech Exp $
#	Manage users and their credentials via htpasswd file
#	Copyright (C) 2005  Python Technology Limited
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
#	02111-1307, USA.
#=======================================================================
package PythonTech::UserDB::HtPasswd;
use strict;

#-----------------------------------------------------------------------
#	Create new user database peer.
#	Configuration:
#	  userFile	File containing user names and crypted passwords
#	  groupFile	(optional) File containing group memberships
#-----------------------------------------------------------------------
sub new {
    my $class = shift;
    my $self = {@_};
    foreach (qw(userFile)) {
	die "$class: $_ not configured\n" unless $self->{$_};
    }
    bless $self, $class;
    return $self;
}

#-----------------------------------------------------------------------
#	Check if user exists in database
#-----------------------------------------------------------------------
sub has_user {
    my($self, $username) = @_;
    $self->_read->{$username} ? 1 : undef;
}

#-----------------------------------------------------------------------
#	Validate the username with presented credentials (password)
#-----------------------------------------------------------------------
sub validate_user {
    my($self, $tusername,$tpassword) = @_;
    $tusername ne ''
	or die "BADUSER: No username given\n";
    my($username) = $tusername =~ /^(\w[-\.\w]*)$/
	or die "BADUSER: Invalid user name `$tusername'\n";
    my $crypted = $self->_read->{$username}
	or die "BADUSER: No such user `$username'\n";
    my $cp2 = crypt($tpassword,substr($crypted,0,2));
    if ($cp2 ne $crypted) {
	die "BADPASSWORD: Incorrect password\n";
    } else {
	return $username;
    }
}

#-----------------------------------------------------------------------
#	Check if a user is in a group.
#	Returns 1 if group exists and contains user, else undef
#-----------------------------------------------------------------------
sub user_in_group {
    my($self, $username,$groupname) = @_;
    return $self->_read_groups->{$groupname}->{$username};
}

#-----------------------------------------------------------------------
#	Add a user with credentials (password)
#-----------------------------------------------------------------------
sub add_user {
    my($self, $username,$props,$checkOnly) = @_;
    die "Invalid character(s) in username\n"
	unless $username =~ /^([-\w\.]+)$/;
    my $password = $props->{'userPassword'};
    die "No password given for user $username\n"
	unless $password ne '';
    return if $checkOnly;
    # Create random 'salt' as per perlfunc manual page
    my $salt = join '', ('.','/',0..9,'A'..'Z','a'..'z')[rand 64, rand 64];
    my $crypted = crypt($password,$salt);
    my $hash = $self->_read;
    $hash->{$username} = $crypted;
    $self->_write($hash);
}

#-----------------------------------------------------------------------
#	Update file
#-----------------------------------------------------------------------
sub _write {
    my($self, $hash) = @_;
    my $newfile = "$self->{'userFile'}.new$$";
    open(HTPASSWD,'>',$newfile)
	or die "Cannot open $newfile for write: $!\n";
    foreach my $u (sort keys %$hash) {
	print HTPASSWD "$u:$hash->{$u}\n";
    }
    close(HTPASSWD)
	or die "Error closing $newfile: $!\n";
    rename($newfile, $self->{'userFile'})
	or die "Error renaming $newfile: $!\n";
}

#-----------------------------------------------------------------------
#	Read user file and return as hash {username => crypted password}
#-----------------------------------------------------------------------
sub _read {
    my($self) = @_;
    my %hash;
    open(HTPASSWD,'<',$self->{'userFile'}) or
	die ref($self).": Could not open user authentication file\n";
    chomp(my @lines = <HTPASSWD>);
    close(HTPASSWD);
    foreach my $line (@lines) {
	my($u,$c) = split(/:/, $line, 2);
	$hash{$u} = $c;
    }
    return \%hash;
}

#-----------------------------------------------------------------------
#	Read group file and return as hash {group => {user => 1}}
#	Format of file:
#	  group1: user1 user2 user3
#	  group2: user4 user1
#-----------------------------------------------------------------------
sub _read_groups {
    my($self) = @_;
    my %hash;
    die ref($self).": No groupFile configured\n"
	unless $self->{'groupFile'};
    open(HTPASSWD,'<',$self->{'groupFile'}) or
	die ref($self).": Could not open group file\n";
    chomp(my @lines = <HTPASSWD>);
    close(HTPASSWD);
    foreach my $line (@lines) {
	my($g,$ul) = split(/:\s+/, $line, 2);
	my @u = split /\s+/, $ul;
	$hash{$g} = {map {$_=>1} @u};
    }
    return \%hash;
}

1;
